@@ -28,13 +28,38 @@
 #include "seahorse-gpgmex.h"
 #include "seahorse-operation.h"
 #include "seahorse-hkp-source.h"
-#include "seahorse-multi-source.h"
 #include "seahorse-util.h"
-#include "seahorse-key.h"
-#include "seahorse-key-pair.h"
+#include "seahorse-pgp-key.h"
+#include "seahorse-gconf.h"
 
 #ifdef WITH_HKP
 
+/* Override the DEBUG_HKP_ENABLE switch here */
+/* #define DEBUG_HKP_ENABLE 1 */
+
+#ifndef DEBUG_HKP_ENABLE
+#if _DEBUG
+#define DEBUG_HKP_ENABLE 1
+#else
+#define DEBUG_HKP_ENABLE 0
+#endif
+#endif
+
+#if DEBUG_HKP_ENABLE
+#define DEBUG_HKP(x) g_printerr x
+#define DEBUG_HEADER(a,b,c) dump_soup_header (a,b,c)
+#define DEBUG_REQUEST(a) dump_soup_request (a)
+#define DEBUG_RESPONSE(a) dump_soup_response (a)
+#else
+#define DEBUG_HKP(x) 
+#define DEBUG_HEADER(a,b,c)
+#define DEBUG_REQUEST(a)
+#define DEBUG_RESPONSE(a)
+#endif
+
+#define PGP_KEY_BEGIN   "-----BEGIN PGP PUBLIC KEY BLOCK-----"
+#define PGP_KEY_END     "-----END PGP PUBLIC KEY BLOCK-----"
+
 /* Amount of keys to load in a batch */
 #define DEFAULT_LOAD_BATCH 30
 
@@ -45,6 +70,13 @@
     
 #define HKP_ERROR_DOMAIN (get_hkp_error_domain())
 
+#define GCONF_USE_HTTP_PROXY    "/system/http_proxy/use_http_proxy"
+#define GCONF_HTTP_PROXY_HOST   "/system/http_proxy/host"
+#define GCONF_PROXY_PORT        "/system/http_proxy/port"
+#define GCONF_USE_AUTH          "/system/http_proxy/use_authentication"
+#define GCONF_AUTH_USER         "/system/http_proxy/authentication_user"
+#define GCONF_AUTH_PASS         "/system/http_proxy/authentication_password"
+
 static GQuark
 get_hkp_error_domain ()
 {
@@ -72,7 +104,7 @@ get_http_server_address (SeahorseKeySource *src)
     return t;
 }
 
-#ifdef _DEBUG
+#ifdef DEBUG_HKP_ENABLE
 
 static void
 dump_soup_header (const gchar *name, const gchar *value, gpointer user_data)
@@ -95,7 +127,7 @@ dump_soup_request (SoupMessage *msg)
     
     if (msg->request.body) {
         t = g_strndup (msg->request.body, msg->request.length);
-        g_printerr ("request: %s\n", t ? t : "");
+        g_printerr ("request: %s\n", t);
         g_free (t);
     }    
 }
@@ -111,12 +143,12 @@ dump_soup_response (SoupMessage *msg)
     
     if (msg->response.body) {
         t = g_strndup (msg->response.body, msg->response.length);
-        g_printerr ("response: %s\n", t ? t : "");
+        g_printerr ("response: %s\n", t);
         g_free (t);
     }    
 }
 
-#endif
+#endif /* DEBUG_HKP_ENABLE */
 
 /* -----------------------------------------------------------------------------
  *  HKP OPERATION     
@@ -132,7 +164,7 @@ dump_soup_response (SoupMessage *msg)
 typedef gboolean (*OpHKPCallback)   (SeahorseOperation *op, SoupMessage *message);
     
 DECLARE_OPERATION (HKP, hkp)
-    SeahorseHKPSource *hsrc;        /* The source to add keys to */
+    SeahorseHKPSource *hsrc;        /* The source  */
     SoupSession *session;           /* The HTTP session */
     guint total;                    /* Number of requests queued */
     guint requests;                 /* Number of requests remaining */
@@ -143,7 +175,42 @@ IMPLEMENT_OPERATION (HKP, hkp)
 static void 
 seahorse_hkp_operation_init (SeahorseHKPOperation *hop)
 {
-    hop->session = soup_session_async_new ();
+    SoupUri *uri;
+    gchar *host;
+    gchar *suri;
+    
+    if (seahorse_gconf_get_boolean (GCONF_USE_HTTP_PROXY)) {
+        
+        host = seahorse_gconf_get_string (GCONF_HTTP_PROXY_HOST);
+        if (host) {
+            
+            suri = g_strdup_printf ("http://%s/", host);
+            g_free (host);
+            
+            uri = soup_uri_new (suri);
+            
+            if (!uri) {
+                g_warning ("creation of SoupUri from '%s' failed", suri);
+                
+            } else {
+                
+                uri->port = seahorse_gconf_get_integer (GCONF_PROXY_PORT);
+                
+                if (seahorse_gconf_get_boolean (GCONF_USE_AUTH)) {
+                    uri->user = seahorse_gconf_get_string (GCONF_AUTH_USER);
+                    uri->passwd = seahorse_gconf_get_string (GCONF_AUTH_PASS);
+                }
+                
+                hop->session = soup_session_async_new_with_options (SOUP_SESSION_PROXY_URI, uri, NULL);
+            }
+
+            g_free (suri);
+        }
+    }
+    
+    /* Without a proxy */
+    if (!hop->session)
+        hop->session = soup_session_async_new ();
 }
 
 static void 
@@ -180,7 +247,7 @@ seahorse_hkp_operation_cancel (SeahorseOperation *operation)
 {
     SeahorseHKPOperation *hop;
     
-    g_return_if_fail (SEAHORSE_IS_HKP_OPERATION (operation));
+    g_assert (SEAHORSE_IS_HKP_OPERATION (operation));
     hop = SEAHORSE_HKP_OPERATION (operation);
     
     if (hop->session != NULL) 
@@ -200,6 +267,8 @@ dehtmlize(gchar *line)
      */
     int parsedindex = 0;
     gchar *parsed = line;
+    
+    g_return_if_fail (line);
 
     while (*line != '\0') {
         switch(*line) {
@@ -271,25 +340,23 @@ fail_hkp_operation (SeahorseHKPOperation *hop, SoupMessage *msg, const gchar *te
     } else if (msg) {
         /* Make the body lower case, and no tags */
         t = g_strndup (msg->response.body, msg->response.length);
-        if (t) {
+        if (t != NULL) {
             dehtmlize (t);        
             seahorse_util_string_lower (t);
+        }
 
-            if (strstr (t, "no keys"))
-                error = g_error_new (HKP_ERROR_DOMAIN, 0, _("No matching keys found on server '%s'."), server);
-            else if (strstr (t, "too many"))
-                error = g_error_new (HKP_ERROR_DOMAIN, 0, _("Search was not specific enough. Server '%s' found too many keys."), server);
-            
-            g_free (t);
-        } 
-
-        if (!error)
+        if (t && strstr (t, "no keys"))
+            error = g_error_new (HKP_ERROR_DOMAIN, 0, _("No matching keys found on server '%s'."), server);
+        else if (t && strstr (t, "too many"))
+            error = g_error_new (HKP_ERROR_DOMAIN, 0, _("Search was not specific enough. Server '%s' found too many keys."), server);
+        else 
             error = g_error_new (HKP_ERROR_DOMAIN, msg->status_code, _("Couldn't communicate with server '%s': %s"),
                                  server, soup_status_get_phrase (msg->status_code));
+        g_free (t);
     } else {
 
         /* We should always have msg or text */
-        g_return_if_reached ();
+        g_assert (FALSE);
     }        
 
     seahorse_operation_mark_done (SEAHORSE_OPERATION (hop), FALSE, error);
@@ -460,10 +527,8 @@ refresh_callback (SoupMessage *msg, SeahorseHKPOperation *hop)
     GList *keys, *k;
     gchar *t;
     
-#ifdef _DEBUG
-    g_printerr ("[hkp] Search Result:\n");
-    dump_soup_response (msg);
-#endif      
+    DEBUG_HKP (("[hkp] Search Result:\n"));
+    DEBUG_RESPONSE (msg);
     
     if (SOUP_MESSAGE_IS_ERROR (msg)) {
         fail_hkp_operation (hop, msg, NULL);
@@ -498,10 +563,7 @@ get_send_result (const gchar *response, unsigned int length)
     gboolean is_error = FALSE;
     
     text = g_strndup (response, length);
-    if (!text)
-        return g_strdup ("");
-    
-    lines = g_strsplit (text, "\n", 0); 
+    lines = g_strsplit (text, "\n", 0);    
 
     for (l = lines; *l; l++) {
 
@@ -537,10 +599,8 @@ send_callback (SoupMessage *msg, SeahorseHKPOperation *hop)
 {
     gchar *errmsg;
 
-#ifdef _DEBUG
-    g_printerr ("[hkp] Send Result:\n");
-    dump_soup_response (msg);
-#endif      
+    DEBUG_HKP (("[hkp] Send Result:\n"));
+    DEBUG_RESPONSE (msg);
     
     if (SOUP_MESSAGE_IS_ERROR (msg)) {
         fail_hkp_operation (hop, msg, NULL);
@@ -564,10 +624,32 @@ send_callback (SoupMessage *msg, SeahorseHKPOperation *hop)
                                           hop->requests, hop->total);
 }
 
+gboolean
+detect_key (const gchar *text, gint len, const gchar **start, const gchar **end)
+{
+    const gchar *t;
+
+    if (len == -1)
+        len = strlen (text);
+    
+    /* Find the first of the headers */
+    if((t = g_strstr_len (text, len, PGP_KEY_BEGIN)) == NULL)
+        return FALSE;
+    if (start)
+        *start = t;
+        
+    /* Find the end of that block */
+    if((t = g_strstr_len (t, len - (t - text), PGP_KEY_END)) == NULL)
+        return FALSE;
+    if (end)
+        *end = t;
+    
+    return TRUE; 
+}
+
 static void 
 get_callback (SoupMessage *msg, SeahorseHKPOperation *hop) 
 {
-    SeahorseTextType type;
     const gchar *start;
     const gchar *end;
     gpgme_data_t data;
@@ -575,10 +657,8 @@ get_callback (SoupMessage *msg, SeahorseHKPOperation *hop)
     guint len;
     int r;
     
-#ifdef _DEBUG
-    g_printerr ("[hkp] Get Result:\n");
-    dump_soup_response (msg);
-#endif      
+    DEBUG_HKP (("[hkp] Get Result:\n"));
+    DEBUG_RESPONSE (msg);
     
     if (SOUP_MESSAGE_IS_ERROR (msg)) {
         fail_hkp_operation (hop, msg, NULL);
@@ -588,26 +668,24 @@ get_callback (SoupMessage *msg, SeahorseHKPOperation *hop)
     end = text = msg->response.body;
     len = msg->response.length;
     
-    do {
+    for (;;) {
 
         len -= end - text;
         text = end;
         
-        type = seahorse_util_detect_text (text, len, &start, &end);
-
+        if(!detect_key (text, len, &start, &end))
+            break;
+        
         /* Any key blocks get written to our result data */
-        if (type == SEAHORSE_TEXT_TYPE_KEY) {
-            data = (gpgme_data_t)g_object_get_data (G_OBJECT (hop), "result");
-            g_return_if_fail (data != NULL);
+        data = (gpgme_data_t)seahorse_operation_get_result (SEAHORSE_OPERATION (hop));
+        g_return_if_fail (data != NULL);
             
-            r = gpgme_data_write (data, start, end - start);
-            g_return_if_fail (r != -1);
+        r = gpgme_data_write (data, start, end - start);
+        g_return_if_fail (r != -1);
 
-            r = gpgme_data_write (data, "\n", 1);
-            g_return_if_fail (r != -1);
-        }
-
-    } while (type != SEAHORSE_TEXT_TYPE_NONE);        
+        r = gpgme_data_write (data, "\n", 1);
+        g_return_if_fail (r != -1);
+    }
         
     if (--hop->requests <= 0)
         seahorse_operation_mark_done (SEAHORSE_OPERATION (hop), FALSE, NULL);
@@ -624,7 +702,8 @@ get_callback (SoupMessage *msg, SeahorseHKPOperation *hop)
 static void seahorse_hkp_source_class_init (SeahorseHKPSourceClass *klass);
 
 /* SeahorseKeySource methods */
-static SeahorseOperation*  seahorse_hkp_source_refresh    (SeahorseKeySource *src,
+static SeahorseOperation*  seahorse_hkp_source_load       (SeahorseKeySource *src,
+                                                           SeahorseKeySourceLoad load,
                                                            const gchar *key);
 static SeahorseOperation*  seahorse_hkp_source_import     (SeahorseKeySource *sksrc, 
                                                            gpgme_data_t data);
@@ -662,7 +741,7 @@ seahorse_hkp_source_class_init (SeahorseHKPSourceClass *klass)
     SeahorseKeySourceClass *key_class;
    
     key_class = SEAHORSE_KEY_SOURCE_CLASS (klass);
-    key_class->refresh = seahorse_hkp_source_refresh;
+    key_class->load = seahorse_hkp_source_load;
     key_class->import = seahorse_hkp_source_import;
     key_class->export = seahorse_hkp_source_export;
 
@@ -670,7 +749,8 @@ seahorse_hkp_source_class_init (SeahorseHKPSourceClass *klass)
 }
 
 static SeahorseOperation*
-seahorse_hkp_source_refresh (SeahorseKeySource *src, const gchar *key)
+seahorse_hkp_source_load (SeahorseKeySource *src, SeahorseKeySourceLoad load,
+                          const gchar *match)
 {
     SeahorseOperation *op;
     SeahorseHKPOperation *hop;
@@ -678,35 +758,27 @@ seahorse_hkp_source_refresh (SeahorseKeySource *src, const gchar *key)
     gchar *pattern = NULL;
     gchar *t, *uri, *server;
     
-    g_return_val_if_fail (SEAHORSE_IS_KEY_SOURCE (src), NULL);
-    g_return_val_if_fail (SEAHORSE_IS_HKP_SOURCE (src), NULL);
-    g_return_val_if_fail (key != NULL, NULL);
+    g_assert (SEAHORSE_IS_KEY_SOURCE (src));
+    g_assert (SEAHORSE_IS_HKP_SOURCE (src));
     
-    op = parent_class->refresh (src, key);
+    op = parent_class->load (src, load, match);
     if (op != NULL)
         return op;
 
-    /* No way to find new keys */
-    if (g_str_equal (key, SEAHORSE_KEY_SOURCE_NEW))
+    /* No way to find new all or new keys */
+    if (load == SKSRC_LOAD_NEW || load == SKSRC_LOAD_ALL)
         return seahorse_operation_new_complete (NULL);
         
-    /* All keys, parent_class will have cleared */
-    else if(g_str_equal (key, SEAHORSE_KEY_SOURCE_ALL)) {
-        
-        g_object_get (src, "pattern", &pattern, NULL);
-        g_return_val_if_fail (pattern && pattern[0], NULL);
-        
-        t = soup_uri_encode (pattern, "+=/\\()");
-        g_free (pattern);
-        pattern = t;
+    else if(load == SKSRC_LOAD_SEARCH)
+        pattern = soup_uri_encode (match, "+=/\\()");
         
     /* Load a specific key */
-    } else {
-
+    else if(load == SKSRC_LOAD_KEY) 
         /* TODO: Does this actually work? */
-        pattern = soup_uri_encode (key, NULL);    
-    } 
-        
+        pattern = soup_uri_encode (match, NULL);    
+
+    g_return_val_if_fail (pattern != NULL, NULL);
+    
     hop = setup_hkp_operation (SEAHORSE_HKP_SOURCE (src));
     
     server = get_http_server_address (src);
@@ -722,10 +794,8 @@ seahorse_hkp_source_refresh (SeahorseKeySource *src, const gchar *key)
     soup_session_queue_message (hop->session, message, 
                                 (SoupMessageCallbackFn)refresh_callback, hop);
 
-#ifdef _DEBUG
-    g_printerr ("[hkp] Search Request:\n");
-    dump_soup_request (message);
-#endif      
+    DEBUG_HKP (("[hkp] Search Request:\n"));
+    DEBUG_REQUEST (message);
     
     hop->total = hop->requests = 1;
     t = g_strdup_printf (_("Searching for keys on: %s"), server);
@@ -734,8 +804,9 @@ seahorse_hkp_source_refresh (SeahorseKeySource *src, const gchar *key)
 
     g_free (server);    
     
-    seahorse_server_source_set_operation (SEAHORSE_SERVER_SOURCE (src), 
-                                          SEAHORSE_OPERATION (hop));
+    seahorse_server_source_take_operation (SEAHORSE_SERVER_SOURCE (src), 
+                                           SEAHORSE_OPERATION (hop));
+    g_object_ref (hop);
     return SEAHORSE_OPERATION (hop);
 }
 
@@ -751,7 +822,7 @@ seahorse_hkp_source_import (SeahorseKeySource *sksrc, gpgme_data_t data)
     GSList *l;
     guint len;
     
-    g_return_val_if_fail (SEAHORSE_IS_HKP_SOURCE (sksrc), NULL);
+    g_assert (SEAHORSE_IS_HKP_SOURCE (sksrc));
     hsrc = SEAHORSE_HKP_SOURCE (sksrc);
     
     for (;;) {
@@ -781,7 +852,7 @@ seahorse_hkp_source_import (SeahorseKeySource *sksrc, gpgme_data_t data)
     hop = setup_hkp_operation (hsrc);
     
     for (l = keydata; l; l = g_slist_next (l)) {
-        g_return_val_if_fail (l->data != NULL, FALSE);
+        g_assert (l->data != NULL);
         t = soup_uri_encode((gchar*)(l->data), "+=/\\()");
 
         key = g_strdup_printf ("keytext=%s", t);
@@ -795,10 +866,8 @@ seahorse_hkp_source_import (SeahorseKeySource *sksrc, gpgme_data_t data)
                                     (SoupMessageCallbackFn)send_callback, hop);
         hop->requests++;
 
-#ifdef _DEBUG
-        g_printerr ("[hkp] Send Request:\n");
-        dump_soup_request (message);
-#endif      
+        DEBUG_HKP (("[hkp] Send Request:\n"));
+        DEBUG_REQUEST (message);
     }
 
     hop->total = hop->requests;
@@ -839,20 +908,20 @@ seahorse_hkp_source_export (SeahorseKeySource *sksrc, GList *keys,
 
     if (data) {
         /* Note that we don't auto-free this */
-        g_object_set_data (G_OBJECT (hop), "result", data);
+        seahorse_operation_mark_result (SEAHORSE_OPERATION (hop), data, NULL);
     } else {
         /* But when we auto create a data object then we free it */
         gpgme_data_new (&data);
         g_return_val_if_fail (data != NULL, NULL);
-        g_object_set_data_full (G_OBJECT (hop), "result", data, 
-                                (GDestroyNotify)gpgme_data_release);
+        seahorse_operation_mark_result (SEAHORSE_OPERATION (hop), data, 
+                                        (GDestroyNotify)gpgme_data_release);
     }
     
     for ( ; keys; keys = g_list_next (keys)) {
-        g_return_val_if_fail (SEAHORSE_IS_KEY (keys->data), NULL);
+        g_assert (SEAHORSE_IS_KEY (keys->data));
 
         /* Get the key id and limit it to 8 characters */
-        fpr = seahorse_key_get_id (SEAHORSE_KEY (keys->data)->key);
+        fpr = seahorse_key_get_keyid (SEAHORSE_KEY (keys->data));
         l = strlen (fpr);
         if (l > 8)
             fpr += (l - 8);
@@ -869,10 +938,8 @@ seahorse_hkp_source_export (SeahorseKeySource *sksrc, GList *keys,
 
         hop->requests++;
 
-#ifdef _DEBUG
-        g_printerr ("[hkp] Get Request:\n");
-        dump_soup_request (message);
-#endif              
+        DEBUG_HKP (("[hkp] Get Request:\n"));
+        DEBUG_REQUEST (message);
     }
     
     hop->total = hop->requests;
@@ -886,24 +953,19 @@ seahorse_hkp_source_export (SeahorseKeySource *sksrc, GList *keys,
 
 /**
  * seahorse_hkp_source_new
- * @locsrc: Local source to base this key source off of
- * @server: The server to connect to 
- * @pattern: The pattern to search for
+ * @uri: The server to connect to 
  * 
  * Creates a new key source for an HKP PGP server.
  * 
  * Returns: A new HKP Key Source
  */
 SeahorseHKPSource*   
-seahorse_hkp_source_new (SeahorseKeySource *locsrc, const gchar *server,
-                          const gchar *pattern)
+seahorse_hkp_source_new (const gchar *uri, const gchar *host)
 {
-    g_return_val_if_fail (SEAHORSE_IS_KEY_SOURCE (locsrc) && 
-                          !SEAHORSE_IS_SERVER_SOURCE (locsrc), NULL);
-    g_return_val_if_fail (server && server[0], NULL);
-
-    return g_object_new (SEAHORSE_TYPE_HKP_SOURCE, "local-source", locsrc,
-                         "key-server", server, "pattern", pattern, NULL);
+    g_return_val_if_fail (seahorse_hkp_is_valid_uri (uri), NULL);
+    g_return_val_if_fail (host && *host, NULL);
+    return g_object_new (SEAHORSE_TYPE_HKP_SOURCE, "uri", uri,
+                         "key-server", host, NULL);
 }
 
 /**
@@ -919,6 +981,8 @@ seahorse_hkp_is_valid_uri (const gchar *uri)
     gboolean ret = FALSE;
     gchar *t;
     
+    g_return_val_if_fail (uri && uri[0], FALSE);
+    
     /* Replace 'hkp' with 'http' at the beginning of the URI */
     if (strncasecmp (uri, "hkp:", 4) == 0) {
         t = g_strdup_printf("http:%s", uri + 4);
