@@ -19,14 +19,21 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
 #include <gnome.h>
 
 #include "seahorse-key-store.h"
+#include "seahorse-pgp-key.h"
 #include "seahorse-gconf.h"
+#include "seahorse-gtkstock.h"
+
+#ifdef WITH_SSH
+#include "seahorse-ssh-key.h"
+#endif 
 
 enum {
 	PROP_0,
-	PROP_KEY_SOURCE,
+	PROP_KEYSET,
     PROP_MODE,
     PROP_FILTER
 };
@@ -89,17 +96,18 @@ static void	     seahorse_key_store_changed (SeahorseKeyStore   *skstore,
                                              SeahorseKeyChange  change);
 
 /* Key source signals */
-static void seahorse_key_store_key_added    (SeahorseKeySource  *sksrc,
+static void seahorse_key_store_key_added    (SeahorseKeyset     *skset,
                                              SeahorseKey        *skey,
                                              SeahorseKeyStore   *skstore);
-static void seahorse_key_store_key_removed  (SeahorseKeySource  *sksrc,
+static void seahorse_key_store_key_removed  (SeahorseKeyset     *skset,
                                              SeahorseKey        *skey,
+                                             gpointer           closure,
+                                             SeahorseKeyStore   *skstore);
+static void	seahorse_key_store_key_changed	(SeahorseKeyset     *skset,
+                                             SeahorseKey		*skey,
+                                             SeahorseKeyChange	change,
+							                 SeahorseKeyRow		*skrow,
                                              SeahorseKeyStore   *skstore);
-                             
-/* Key signals */
-static void	seahorse_key_store_key_changed		(SeahorseKey		*skey,
-							 SeahorseKeyChange	change,
-							 SeahorseKeyRow		*skrow);
 /* Key Row methods */
 static void	seahorse_key_row_add			(SeahorseKeyStore	*skstore,
                                              GtkTreeIter        *iter,
@@ -165,10 +173,10 @@ seahorse_key_store_class_init (SeahorseKeyStoreClass *klass)
     klass->col_types = col_types;
     klass->col_ids = col_ids;
 	
-	g_object_class_install_property (gobject_class, PROP_KEY_SOURCE,
-		g_param_spec_object ("key-source", "Seahorse Key Source",
+	g_object_class_install_property (gobject_class, PROP_KEYSET,
+		g_param_spec_object ("keyset", "Seahorse Keyset",
 				     "Current Seahorse Key Source to use",
-				     SEAHORSE_TYPE_KEY_SOURCE, G_PARAM_READWRITE));
+				     SEAHORSE_TYPE_KEYSET, G_PARAM_READWRITE));
                     
     g_object_class_install_property (gobject_class, PROP_MODE,
         g_param_spec_uint ("mode", "Key Store Mode",
@@ -223,13 +231,15 @@ seahorse_key_store_dispose (GObject *gobject)
 
     skstore = SEAHORSE_KEY_STORE (gobject);  
 
-    if (skstore->sksrc) {
-        g_signal_handlers_disconnect_by_func (skstore->sksrc, 
+    if (skstore->skset) {
+        g_signal_handlers_disconnect_by_func (skstore->skset, 
                                 seahorse_key_store_key_added, skstore);
-        g_signal_handlers_disconnect_by_func (skstore->sksrc, 
+        g_signal_handlers_disconnect_by_func (skstore->skset, 
                                 seahorse_key_store_key_removed, skstore);
-        g_object_unref (skstore->sksrc);        
-        skstore->sksrc = NULL;
+        g_signal_handlers_disconnect_by_func (skstore->skset, 
+                                seahorse_key_store_key_changed, skstore);
+        g_object_unref (skstore->skset);        
+        skstore->skset = NULL;
     }
     
     G_OBJECT_CLASS (parent_class)->dispose (gobject);
@@ -241,7 +251,7 @@ seahorse_key_store_finalize (GObject *gobject)
 	SeahorseKeyStore *skstore;
 	
 	skstore = SEAHORSE_KEY_STORE (gobject);
-    g_assert (skstore->sksrc == NULL);
+    g_assert (skstore->skset == NULL);
     	
     /* These were allocated in the constructor */
     g_object_unref (skstore->priv->sort);
@@ -260,7 +270,8 @@ seahorse_key_store_finalize (GObject *gobject)
 static gboolean
 refilter_now(SeahorseKeyStore* skstore)
 {
-    gtk_tree_model_filter_refilter (skstore->priv->filter);
+    seahorse_keyset_refresh (skstore->skset);
+    gtk_tree_model_filter_refilter (skstore->priv->filter);    
     skstore->priv->filter_stag = 0;
     return FALSE;
 }
@@ -284,15 +295,16 @@ seahorse_key_store_set_property (GObject *gobject, guint prop_id,
 	skstore = SEAHORSE_KEY_STORE (gobject);
 	
 	switch (prop_id) {
-		/* Connects to key source signals */
-		case PROP_KEY_SOURCE:
-            g_return_if_fail (skstore->sksrc == NULL);
-			skstore->sksrc = g_value_get_object (value);
-			g_object_ref (skstore->sksrc);
-            g_signal_connect_after (skstore->sksrc, "added",
+        case PROP_KEYSET:
+            g_assert (skstore->skset == NULL);
+            skstore->skset = g_value_get_object (value);
+            g_object_ref (skstore->skset);
+            g_signal_connect_after (skstore->skset, "added",
                 G_CALLBACK (seahorse_key_store_key_added), skstore);
-            g_signal_connect_after (skstore->sksrc, "removed",
+            g_signal_connect_after (skstore->skset, "removed",
                 G_CALLBACK (seahorse_key_store_key_removed), skstore);
+            g_signal_connect_after (skstore->skset, "changed",
+                G_CALLBACK (seahorse_key_store_key_changed), skstore);
 			break;
 			
 		/* The filtering mode */
@@ -335,8 +347,8 @@ seahorse_key_store_get_property (GObject *gobject, guint prop_id,
 	skstore = SEAHORSE_KEY_STORE (gobject);
 	
 	switch (prop_id) {
-		case PROP_KEY_SOURCE:
-			g_value_set_object (value, skstore->sksrc);
+		case PROP_KEYSET:
+			g_value_set_object (value, skstore->skset);
 			break;
 			
 		/* The filtering mode */
@@ -371,14 +383,26 @@ static void
 seahorse_key_store_set (SeahorseKeyStore *skstore, SeahorseKey *skey, 
                         guint uid, GtkTreeIter *iter)
 {
-    gchar *userid = seahorse_key_get_userid (skey, uid);
-    gboolean pair = SEAHORSE_IS_KEY_PAIR(skey);
+    gchar *userid = seahorse_key_get_name (skey, uid);
+    gboolean sec = seahorse_key_get_etype (skey) == SKEY_PRIVATE;
+    const gchar *stockid = NULL;
+
+#ifdef WITH_SSH    
+    if (seahorse_key_get_ktype (skey) == SKEY_SSH)
+        stockid = SEAHORSE_STOCK_KEY_SSH;
+    else
+#endif
+    if(seahorse_key_get_ktype (skey) == SKEY_PGP) {
+        if (uid == 0)
+            stockid = sec ? SEAHORSE_STOCK_SECRET : SEAHORSE_STOCK_KEY;
+    }
+    
 	gtk_tree_store_set (GTK_TREE_STORE (skstore), iter,
         KEY_STORE_CHECK, FALSE,
-        KEY_STORE_PAIR, uid == 0 ? pair : FALSE,
-        KEY_STORE_NOTPAIR, uid == 0 ? !pair : FALSE,
+        KEY_STORE_PAIR, uid == 0 ? sec : FALSE,
+        KEY_STORE_STOCK_ID, stockid,
 		KEY_STORE_NAME, userid,
-		KEY_STORE_KEYID, seahorse_key_get_keyid (skey, 0),
+		KEY_STORE_KEYID, seahorse_key_get_short_keyid (skey),
         KEY_STORE_UID, uid, -1);
     g_free (userid);
 }
@@ -400,10 +424,10 @@ seahorse_key_store_changed (SeahorseKeyStore *skstore, SeahorseKey *skey, guint
 
 /* Appends @skey and all uids */
 static void
-seahorse_key_store_key_added (SeahorseKeySource *sksrc, SeahorseKey *skey, SeahorseKeyStore *skstore)
+seahorse_key_store_key_added (SeahorseKeyset *skset, SeahorseKey *skey, SeahorseKeyStore *skstore)
 {
 	GtkTreeIter iter;
-	guint i, uids = seahorse_key_get_num_uids (skey);
+	guint i, uids = seahorse_key_get_num_names (skey);
  
     for (i = 0; i < uids; i++) {
     	if(!SEAHORSE_KEY_STORE_GET_CLASS (skstore)->append (skstore, skey, i, &iter))
@@ -413,16 +437,18 @@ seahorse_key_store_key_added (SeahorseKeySource *sksrc, SeahorseKey *skey, Seaho
 
 /* Removes @skrow */
 static void
-seahorse_key_store_key_removed (SeahorseKeySource *sksrc, SeahorseKey *skey, SeahorseKeyStore *skstore)
+seahorse_key_store_key_removed (SeahorseKeyset *skset, SeahorseKey *skey, 
+                                gpointer closure, SeahorseKeyStore *skstore)
 {
-    SeahorseKeyRow *skrow = (SeahorseKeyRow*)g_hash_table_lookup (skstore->priv->rows, skey);
-    if (skrow)
-	    seahorse_key_row_remove_all (skrow);
+	SeahorseKeyRow *skrow = (SeahorseKeyRow*)g_hash_table_lookup (skstore->priv->rows, skey);
+	if(skrow)
+		seahorse_key_row_remove_all (skrow);
 }
 
 /* Calls virtual |changed| for all relevant uids. adds new uids if necessary */
 static void
-seahorse_key_store_key_changed (SeahorseKey *skey, SeahorseKeyChange change, SeahorseKeyRow *skrow)
+seahorse_key_store_key_changed (SeahorseKeyset *skset, SeahorseKey *skey, 
+                                SeahorseKeyChange change, SeahorseKeyRow *skrow, SeahorseKeyStore *skstore)
 {
     guint i, uid, old_uids, num_uids;
     GtkTreeIter first;
@@ -430,10 +456,10 @@ seahorse_key_store_key_changed (SeahorseKey *skey, SeahorseKeyChange change, Sea
     GtkTreePath *path;
     
     old_uids = 0;
-    num_uids = seahorse_key_get_num_uids (skey);
+    num_uids = seahorse_key_get_num_names (skey);
     
     for (i = 0; i < skrow->refs->len; i++) {
-        g_return_if_fail (g_ptr_array_index (skrow->refs, i) != NULL);
+        g_assert (g_ptr_array_index (skrow->refs, i) != NULL);
         path = gtk_tree_row_reference_get_path ((GtkTreeRowReference*)g_ptr_array_index (skrow->refs, i));
         g_return_if_fail (gtk_tree_model_get_iter (GTK_TREE_MODEL (skrow->skstore), &iter, path));
         gtk_tree_path_free (path);        
@@ -565,7 +591,7 @@ check_toggled(GtkCellRendererToggle *cellrenderertoggle, gchar *path, gpointer u
     GValue v;
 
     memset (&v, 0, sizeof(v));
-    g_return_if_fail (path != NULL);
+    g_assert (path != NULL);
     g_return_if_fail (gtk_tree_model_get_iter_from_string (fmodel, &iter, path));
     
     /* We get notified in filtered coordinates, we have to convert those to base */
@@ -594,7 +620,7 @@ row_activated (GtkTreeView *treeview, GtkTreePath *path, GtkTreeViewColumn *arg2
     GValue v;
 
     memset (&v, 0, sizeof(v));
-    g_return_if_fail (path != NULL);
+    g_assert (path != NULL);
     g_return_if_fail (gtk_tree_model_get_iter (fmodel, &iter, path));
     
     /* We get notified in filtered coordinates, we have to convert those to base */
@@ -629,8 +655,7 @@ seahorse_key_row_add (SeahorseKeyStore *skstore, GtkTreeIter *iter, SeahorseKey
         skrow->skey = skey;
         g_object_ref (skey);
 
-        g_signal_connect_after (skrow->skey, "changed",
-                                G_CALLBACK (seahorse_key_store_key_changed), skrow);
+        seahorse_keyset_set_closure (skstore->skset, skey, skrow);
 
         /* Put it in our row cache */
         g_hash_table_replace (skstore->priv->rows, skey, skrow);
@@ -649,12 +674,10 @@ seahorse_key_row_free (SeahorseKeyRow *skrow)
     guint i;
     
     /* Unref key */
-    g_signal_handlers_disconnect_by_func (skrow->skey,
-                                          seahorse_key_store_key_changed, skrow);
     g_object_unref (skrow->skey);
   
     for (i = 0; i < skrow->refs->len; i++) {
-        g_return_if_fail (g_ptr_array_index (skrow->refs, i) != NULL);
+        g_assert (g_ptr_array_index (skrow->refs, i) != NULL);
         gtk_tree_row_reference_free ((GtkTreeRowReference*)g_ptr_array_index (skrow->refs, i));
     }
     g_ptr_array_free (skrow->refs, TRUE);
@@ -673,7 +696,7 @@ seahorse_key_row_remove_all (SeahorseKeyRow *skrow)
     guint i;
     
     for (i = 0; i < skrow->refs->len; i++) {
-        g_return_if_fail (g_ptr_array_index (skrow->refs, i) != NULL);
+        g_assert (g_ptr_array_index (skrow->refs, i) != NULL);
         path = gtk_tree_row_reference_get_path ((GtkTreeRowReference*)g_ptr_array_index (skrow->refs, i));
         
         /* Note that removing a parent row could remove it's sub rows, so ... */
@@ -697,7 +720,7 @@ seahorse_key_row_remove (SeahorseKeyRow *skrow, GtkTreeIter *iter)
     p1 = gtk_tree_model_get_path (GTK_TREE_MODEL (skrow->skstore), iter);
     
     for (i = 0; i < skrow->refs->len; i++) {
-        g_return_if_fail (g_ptr_array_index (skrow->refs, i) != NULL);
+        g_assert (g_ptr_array_index (skrow->refs, i) != NULL);
         p2 = gtk_tree_row_reference_get_path ((GtkTreeRowReference*)g_ptr_array_index (skrow->refs, i));
         r = gtk_tree_path_compare (p1, p2);
         gtk_tree_path_free (p2);
@@ -746,43 +769,22 @@ seahorse_key_store_init (SeahorseKeyStore *skstore, GtkTreeView *view)
     
     /* When using key pair icons, we add an icon column */
     if (SEAHORSE_KEY_STORE_GET_CLASS (skstore)->use_icon) {
-    	GtkCellRenderer *secretPixbufRenderer = NULL, *keyPixbufRenderer = NULL;
-    	GdkPixbuf *secret_pixbuf = NULL, *key_pixbuf;
-    	
-    	secret_pixbuf = gdk_pixbuf_new_from_file(PIXMAPSDIR "seahorse-secret.png", NULL);
-    	secretPixbufRenderer = gtk_cell_renderer_pixbuf_new();
-    	g_object_set(secretPixbufRenderer,
-    				 "pixbuf", GDK_PIXBUF(secret_pixbuf),
-    				 NULL);
-    	
-    	key_pixbuf = gdk_pixbuf_new_from_file(PIXMAPSDIR "seahorse-key.png", NULL);
-    	keyPixbufRenderer = gtk_cell_renderer_pixbuf_new();
-    	g_object_set(keyPixbufRenderer,
-    				 "pixbuf", GDK_PIXBUF(key_pixbuf),
-    				 NULL);
-    				 			 
-    	col = gtk_tree_view_column_new_with_attributes ("", secretPixbufRenderer, 
-   												"visible", KEY_STORE_PAIR,
+    	GtkCellRenderer  *renderer = gtk_cell_renderer_pixbuf_new ();
+        g_object_set (renderer, "stock-size", GTK_ICON_SIZE_LARGE_TOOLBAR, NULL);
+    	col = gtk_tree_view_column_new_with_attributes ("", renderer, 
+   												"stock-id", KEY_STORE_STOCK_ID,
    												NULL);
-
-   		gtk_tree_view_column_pack_end(col,
-   									  keyPixbufRenderer,
-   									  FALSE);
-   									  
-   		gtk_tree_view_column_add_attribute(col, keyPixbufRenderer,
-   												"visible",
-   												KEY_STORE_NOTPAIR); 										
+										
         gtk_tree_view_column_set_resizable (col, FALSE);
         gtk_tree_view_append_column (view, col);
         
         gtk_tree_view_column_set_sort_column_id (col, KEY_STORE_PAIR);
-    }
-    
-    col = seahorse_key_store_append_column (view, _("Name"), KEY_STORE_NAME);
-    gtk_tree_view_column_set_sort_column_id (col, KEY_STORE_NAME);
-    
-    col = seahorse_key_store_append_column (view, _("Key ID"), KEY_STORE_KEYID);
-    gtk_tree_view_column_set_sort_column_id (col, KEY_STORE_KEYID);
+      }
+      
+	col = seahorse_key_store_append_column (view, _("Name"), KEY_STORE_NAME);
+	gtk_tree_view_column_set_sort_column_id (col, KEY_STORE_NAME);
+	
+	seahorse_key_store_append_column (view, _("Key ID"), KEY_STORE_KEYID);
 
     if (SEAHORSE_KEY_STORE_GET_CLASS (skstore)->gconf_sort_key) {
         /* Also watch for sort-changed on the store */
@@ -802,22 +804,18 @@ seahorse_key_store_populate (SeahorseKeyStore *skstore)
 {
 	GList *keys, *list = NULL;
 	SeahorseKey *skey;
-	guint count = 1;
-	gdouble length;
 	
     g_return_if_fail (SEAHORSE_IS_KEY_STORE (skstore));
-    g_return_if_fail (SEAHORSE_IS_KEY_SOURCE (skstore->sksrc));
+    g_return_if_fail (SEAHORSE_IS_KEYSET (skstore->skset));
 	
     /* Don't precipitate a load */
-    if (seahorse_key_source_get_count (skstore->sksrc, FALSE) > 0) {
+    if (seahorse_keyset_get_count (skstore->skset) > 0) {
 
-    	keys = list = seahorse_key_source_get_keys (skstore->sksrc, FALSE);
-    	length = g_list_length (list);
+    	keys = list = seahorse_keyset_get_keys (skstore->skset);
 	
     	while (list != NULL && (skey = list->data) != NULL) {
-            seahorse_key_store_key_added (skstore->sksrc, skey, skstore);
+            seahorse_key_store_key_added (skstore->skset, skey, skstore);
     		list = g_list_next (list);
-    		count++;
     	}
      
         g_list_free (keys);
@@ -928,7 +926,7 @@ seahorse_key_store_get_all_keys (GtkTreeView *view)
     g_return_val_if_fail (GTK_IS_TREE_VIEW (view), NULL);
     skstore = key_store_from_model (gtk_tree_view_get_model (view));
     
-    return seahorse_key_source_get_keys (skstore->sksrc, FALSE);
+    return seahorse_keyset_get_keys (skstore->skset);
 }
 
 /**
