@@ -2,6 +2,7 @@
  * Seahorse
  *
  * Copyright (C) 2003 Jacob Perkins
+ * Copyright (C) 2005 Nate Nielsen
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,14 +26,15 @@
 #include "seahorse-op.h"
 #include "seahorse-util.h"
 #include "seahorse-context.h"
-#include "seahorse-key-source.h"
+#include "seahorse-pgp-source.h"
+#include "seahorse-pgp-key.h"
 #include "seahorse-vfs-data.h"
 #include "seahorse-gconf.h"
 
 /* helper function for importing @data. @data will be released.
  * returns number of keys imported or -1. */
 static gint
-import_data (SeahorseKeySource *sksrc, gpgme_data_t data, 
+import_data (SeahorsePGPSource *psrc, gpgme_data_t data, 
              GError **err)
 {
     SeahorseOperation *operation;
@@ -41,7 +43,7 @@ import_data (SeahorseKeySource *sksrc, gpgme_data_t data,
     
     g_return_val_if_fail (!err || !err[0], -1);
 
-    operation = seahorse_key_source_import (sksrc, data);
+    operation = seahorse_key_source_import (SEAHORSE_KEY_SOURCE (psrc), data);
     g_return_val_if_fail (operation != NULL, -1);
     
     seahorse_operation_wait (operation);
@@ -71,7 +73,7 @@ import_data (SeahorseKeySource *sksrc, gpgme_data_t data,
  * Returns: Number of keys imported or -1 if import fails
  **/
 gint
-seahorse_op_import_file (SeahorseKeySource *sksrc, const gchar *path, GError **err)
+seahorse_op_import_file (SeahorsePGPSource *psrc, const gchar *path, GError **err)
 {
 	gpgme_data_t data;
 	gpgme_error_t gerr;
@@ -83,7 +85,7 @@ seahorse_op_import_file (SeahorseKeySource *sksrc, const gchar *path, GError **e
         return -1;
     }
     
-	return import_data (sksrc, data, err);
+	return import_data (psrc, data, err);
 }
 
 /**
@@ -97,7 +99,7 @@ seahorse_op_import_file (SeahorseKeySource *sksrc, const gchar *path, GError **e
  * Returns: Number of keys imported or -1 if import fails
  **/
 gint
-seahorse_op_import_text (SeahorseKeySource *sksrc, const gchar *text, GError **err)
+seahorse_op_import_text (SeahorsePGPSource *psrc, const gchar *text, GError **err)
 {
 	gpgme_data_t data;
     gpgme_error_t gerr;
@@ -111,7 +113,7 @@ seahorse_op_import_text (SeahorseKeySource *sksrc, const gchar *text, GError **e
         g_return_val_if_reached (-1);
     }	
     
-	return import_data (sksrc, data, err);
+	return import_data (psrc, data, err);
 }
 
 /* helper function for exporting @keys. */
@@ -135,7 +137,7 @@ export_data (GList *keys, gboolean complete, gboolean force_armor,
         /* Break off one set (same keysource) */
         next = seahorse_util_keylist_splice (keys);
 
-        g_return_val_if_fail (SEAHORSE_IS_KEY (keys->data), FALSE);
+        g_assert (SEAHORSE_IS_KEY (keys->data));
         skey = SEAHORSE_KEY (keys->data);
 
         /* Export from this key source */        
@@ -205,7 +207,6 @@ seahorse_op_export_file (GList *keys, gboolean complete, const gchar *path,
  *
  * Tries to export @recips to text using seahorse_util_write_data_to_text(),
  * saving any errors in @err. @recips will be released upon completion.
- * ASCII Armor setting of @sctx will be ignored.
  *
  * Returns: The exported text or NULL if the operation fails
  **/
@@ -238,32 +239,28 @@ typedef gpgme_error_t (* EncryptFunc) (gpgme_ctx_t ctx, gpgme_key_t recips[],
 /* helper function for encrypting @plain to @recips using @func. @plain and
  * @keys will be released.*/
 static void
-encrypt_data_common (SeahorseKeySource *sksrc, GList *keys, gpgme_data_t plain, 
+encrypt_data_common (SeahorsePGPSource *psrc, GList *keys, gpgme_data_t plain, 
                      gpgme_data_t cipher, EncryptFunc func, gboolean force_armor, 
                      gpgme_error_t *err)
 {
     SeahorseKey *skey;
     gpgme_key_t *recips;
-    gchar *id;
 
 	/* if don't already have an error, do encrypt */
 	if (GPG_IS_OK (*err)) {
 
         /* Add the default key if set and necessary */
         if (seahorse_gconf_get_boolean (ENCRYPTSELF_KEY)) {
-            id = seahorse_gconf_get_string (DEFAULT_KEY);        
-            if (id != NULL) {
-                skey = seahorse_key_source_get_key (sksrc, id);
-                if (skey != NULL)
-                    keys = g_list_append (keys, skey);
-            }
+            skey = seahorse_context_get_default_key (SCTX_APP ());
+            if (skey != NULL)
+                keys = g_list_append (keys, skey);
         }
 
         /* Make keys into the right format */
         recips = seahorse_util_keylist_to_keys (keys);
         
-        gpgme_set_armor (sksrc->ctx, force_armor || seahorse_gconf_get_boolean (ARMOR_KEY));
-		*err = func (sksrc->ctx, recips, GPGME_ENCRYPT_ALWAYS_TRUST, plain, cipher);
+        gpgme_set_armor (psrc->gctx, force_armor || seahorse_gconf_get_boolean (ARMOR_KEY));
+		*err = func (psrc->gctx, recips, GPGME_ENCRYPT_ALWAYS_TRUST, plain, cipher);
         
         seahorse_util_free_keys (recips);
 	}
@@ -274,21 +271,15 @@ encrypt_data_common (SeahorseKeySource *sksrc, GList *keys, gpgme_data_t plain,
 /* common file encryption helper to encrypt @path to @recips using @func.
  * @keys will be released.*/
 static void
-encrypt_file_common (GList *keys, const gchar *path, const gchar *epath,
-                     EncryptFunc func, gpgme_error_t *err)
+encrypt_file_common (SeahorsePGPSource *psrc, GList *keys, const gchar *path, 
+                     const gchar *epath, EncryptFunc func, gpgme_error_t *err)
 {
-    SeahorseKeySource *sksrc;
 	gpgme_data_t plain, cipher;
 	gpgme_error_t error;
 	
 	if (err == NULL)
 		err = &error;
 
-    /* When S/MIME supported need to make sure all keys from the same source */       
-    g_return_if_fail (keys && SEAHORSE_IS_KEY (keys->data));
-    sksrc = seahorse_key_get_source (SEAHORSE_KEY (keys->data));
-    g_return_if_fail (sksrc != NULL);
-
     plain = seahorse_vfs_data_create (path, SEAHORSE_VFS_READ, err);
     g_return_if_fail (plain != NULL);
     
@@ -298,30 +289,24 @@ encrypt_file_common (GList *keys, const gchar *path, const gchar *epath,
         g_return_if_reached ();
     }
  
-    gpgme_set_textmode (sksrc->ctx, FALSE);
-	encrypt_data_common (sksrc, keys, plain, cipher, func, FALSE, err);
+    gpgme_set_textmode (psrc->gctx, FALSE);
+	encrypt_data_common (psrc, keys, plain, cipher, func, FALSE, err);
 	g_return_if_fail (GPG_IS_OK (*err));
 
     gpgme_data_release (cipher);	
 }
 
 /* common text encryption helper to encrypt @text to @recips using @func.
- * ASCII Armor setting of @sctx is ignored. returns the encrypted text. */
+ * returns the encrypted text. */
 static gchar*
-encrypt_text_common (GList *keys, const gchar *text, EncryptFunc func, 
-                     gpgme_error_t *err)
+encrypt_text_common (SeahorsePGPSource *psrc, GList *keys, const gchar *text, 
+                     EncryptFunc func, gpgme_error_t *err)
 {
-    SeahorseKeySource *sksrc;
 	gpgme_data_t plain, cipher;
 	gpgme_error_t error;
 	
 	if (err == NULL)
 		err = &error;
-
-    /* When S/MIME supported need to make sure all keys from the same source */       
-    g_return_val_if_fail (keys && SEAHORSE_IS_KEY (keys->data), NULL);
-    sksrc = seahorse_key_get_source (SEAHORSE_KEY (keys->data));
-    g_return_val_if_fail (sksrc != NULL, NULL);
     
 	/* new data form text */
 	*err = gpgme_data_new_from_mem (&plain, text, strlen (text), TRUE);
@@ -331,8 +316,8 @@ encrypt_text_common (GList *keys, const gchar *text, EncryptFunc func,
     g_return_val_if_fail (GPG_IS_OK (*err), NULL);
    
 	/* encrypt with armor */
-    gpgme_set_textmode (sksrc->ctx, TRUE);    
-	encrypt_data_common (sksrc, keys, plain, cipher, func, TRUE, err);
+    gpgme_set_textmode (psrc->gctx, TRUE);    
+	encrypt_data_common (psrc, keys, plain, cipher, func, TRUE, err);
 	g_return_val_if_fail (GPG_IS_OK (*err), NULL);
 	
 	return seahorse_util_write_data_to_text (cipher, TRUE);
@@ -351,7 +336,15 @@ void
 seahorse_op_encrypt_file (GList *keys, const gchar *path, const gchar *epath,
                           gpgme_error_t *err)
 {
-	return encrypt_file_common (keys, path, epath, gpgme_op_encrypt, err);
+    SeahorsePGPSource *psrc;
+    
+    /* TODO: When other key types are supported we need to make sure they're
+       all from the same PGP source */
+    g_return_if_fail (keys && SEAHORSE_IS_PGP_KEY (keys->data));
+    psrc = SEAHORSE_PGP_SOURCE (seahorse_key_get_source (SEAHORSE_KEY (keys->data)));
+    g_return_if_fail (psrc != NULL && SEAHORSE_IS_PGP_SOURCE (psrc));
+
+	return encrypt_file_common (psrc, keys, path, epath, gpgme_op_encrypt, err);
 }
 
 /**
@@ -360,36 +353,40 @@ seahorse_op_encrypt_file (GList *keys, const gchar *path, const gchar *epath,
  * @text: Text to encrypt
  * @err: Optional error value
  *
- * Tries to encrypt @text to @recips, saving any errors in @err. The ASCII Armor
- * setting of @sctx will be ignored. @recips will be released upon completion.
+ * Tries to encrypt @text to @recips, saving any errors in @err. @recips will be 
+ * released upon completion.
  *
  * Returns: The encrypted text or NULL if encryption fails
  **/
 gchar*
 seahorse_op_encrypt_text (GList *keys, const gchar *text, gpgme_error_t *err)
 {
-	return encrypt_text_common (keys, text, gpgme_op_encrypt, err);
+    SeahorsePGPSource *psrc;
+
+    /* TODO: When other key types are supported we need to make sure they're
+       all from the same PGP source */
+    g_return_val_if_fail (keys && SEAHORSE_IS_PGP_KEY (keys->data), NULL);
+    psrc = SEAHORSE_PGP_SOURCE (seahorse_key_get_source (SEAHORSE_KEY (keys->data)));
+    g_return_val_if_fail (psrc != NULL && SEAHORSE_IS_PGP_SOURCE (psrc), NULL);
+    
+	return encrypt_text_common (psrc, keys, text, gpgme_op_encrypt, err);
 }
 
 /* Helper function to set a key pair as the signer for its keysource */
 static void
-set_signer (SeahorseKeyPair *signer)
+set_signer (SeahorsePGPSource *psrc, SeahorsePGPKey *signer)
 {
-    SeahorseKeySource *sksrc;
-    
-    sksrc = seahorse_key_get_source (SEAHORSE_KEY (signer));
-    g_return_if_fail (sksrc != NULL);
-    
-    gpgme_signers_clear (sksrc->ctx);
-    gpgme_signers_add (sksrc->ctx, signer->secret);
+    gpgme_signers_clear (psrc->gctx);
+    if (signer)
+        gpgme_signers_add (psrc->gctx, signer->seckey);
 }
 
 /* helper function for signing @plain with @mode. @plain will be released. */
 static void
-sign_data (SeahorseKeySource *sksrc, gpgme_data_t plain, gpgme_data_t sig,
+sign_data (SeahorsePGPSource *psrc, gpgme_data_t plain, gpgme_data_t sig,
            gpgme_sig_mode_t mode, gpgme_error_t *err)
 {
-	*err = gpgme_op_sign (sksrc->ctx, plain, sig, mode);
+	*err = gpgme_op_sign (psrc->gctx, plain, sig, mode);
 	gpgme_data_release (plain);
 }
 
@@ -404,18 +401,21 @@ sign_data (SeahorseKeySource *sksrc, gpgme_data_t plain, gpgme_data_t sig,
  * in @err. 
  **/
 void
-seahorse_op_sign_file (SeahorseKeyPair *signer, const gchar *path, 
+seahorse_op_sign_file (SeahorsePGPKey *signer, const gchar *path, 
                        const gchar *spath, gpgme_error_t *err)
 {
-    SeahorseKeySource *sksrc;
+    SeahorsePGPSource *psrc;
 	gpgme_data_t plain, sig;
 	gpgme_error_t error;
+
+    g_return_if_fail (signer && SEAHORSE_IS_PGP_KEY (signer));
+    g_return_if_fail (seahorse_key_get_flags (SEAHORSE_KEY (signer)) & SKEY_FLAG_CAN_SIGN);
 	
 	if (err == NULL)
 		err = &error;
 
-    sksrc = seahorse_key_get_source (SEAHORSE_KEY (signer));
-    g_return_if_fail (sksrc != NULL);
+    psrc = SEAHORSE_PGP_SOURCE (seahorse_key_get_source (SEAHORSE_KEY (signer)));
+    g_return_if_fail (psrc != NULL && SEAHORSE_IS_PGP_SOURCE (psrc));
     
 	/* new data from file */
     plain = seahorse_vfs_data_create (path, SEAHORSE_VFS_READ, err);
@@ -427,12 +427,12 @@ seahorse_op_sign_file (SeahorseKeyPair *signer, const gchar *path,
         g_return_if_reached ();
     }
   
-    set_signer (signer);
+    set_signer (psrc, signer);
     
 	/* get detached signature */
-    gpgme_set_textmode (sksrc->ctx, FALSE);
-    gpgme_set_armor (sksrc->ctx, seahorse_gconf_get_boolean (ARMOR_KEY));    
-	sign_data (sksrc, plain, sig, GPGME_SIG_MODE_DETACH, err);
+    gpgme_set_textmode (psrc->gctx, FALSE);
+    gpgme_set_armor (psrc->gctx, seahorse_gconf_get_boolean (ARMOR_KEY));    
+	sign_data (psrc, plain, sig, GPGME_SIG_MODE_DETACH, err);
 	g_return_if_fail (GPG_IS_OK (*err));
   
     gpgme_data_release (sig);
@@ -445,25 +445,28 @@ seahorse_op_sign_file (SeahorseKeyPair *signer, const gchar *path,
  * @err: Optional error value
  *
  * Tries to sign @text using a clear text signature, saving any errors in @err.
- * Signing will be done by the default key of @sctx.
+ * Signing will be done by the default key.
  *
  * Returns: The clear signed text or NULL if signing fails
  **/
 gchar*
-seahorse_op_sign_text (SeahorseKeyPair *signer, const gchar *text, 
+seahorse_op_sign_text (SeahorsePGPKey *signer, const gchar *text, 
                        gpgme_error_t *err)
 {
-    SeahorseKeySource *sksrc;
+    SeahorsePGPSource *psrc;
 	gpgme_data_t plain, sig;
 	gpgme_error_t error;
+
+    g_return_val_if_fail (signer && SEAHORSE_IS_PGP_KEY (signer), NULL);
+    g_return_val_if_fail (seahorse_key_get_flags (SEAHORSE_KEY (signer)) & SKEY_FLAG_CAN_SIGN, NULL);
 	
 	if (err == NULL)
 		err = &error;
 
-    sksrc = seahorse_key_get_source (SEAHORSE_KEY (signer));
-    g_return_val_if_fail (sksrc != NULL, NULL);
+    psrc = SEAHORSE_PGP_SOURCE (seahorse_key_get_source (SEAHORSE_KEY (signer)));
+    g_return_val_if_fail (psrc != NULL && SEAHORSE_IS_PGP_SOURCE (psrc), NULL);
     
-    set_signer (signer);
+    set_signer (psrc, signer);
             
 	/* new data from text */
 	*err = gpgme_data_new_from_mem (&plain, text, strlen (text), TRUE);
@@ -473,9 +476,9 @@ seahorse_op_sign_text (SeahorseKeyPair *signer, const gchar *text,
     g_return_val_if_fail (GPG_IS_OK (*err), NULL);    
     
 	/* clear sign data already ignores ASCII Armor */
-    gpgme_set_textmode (sksrc->ctx, TRUE);
-    gpgme_set_armor (sksrc->ctx, TRUE);
-	sign_data (sksrc, plain, sig, GPGME_SIG_MODE_CLEAR, err);
+    gpgme_set_textmode (psrc->gctx, TRUE);
+    gpgme_set_armor (psrc->gctx, TRUE);
+	sign_data (psrc, plain, sig, GPGME_SIG_MODE_CLEAR, err);
 	g_return_val_if_fail (GPG_IS_OK (*err), NULL);
 	
 	return seahorse_util_write_data_to_text (sig, TRUE);
@@ -490,15 +493,26 @@ seahorse_op_sign_text (SeahorseKeyPair *signer, const gchar *text,
  * @err: Optional error value
  *
  * Tries to encrypt and sign the file @path to @recips, saving any errors in @err.
- * Signing will be done with the default key of @sctx. @recips will be released
+ * Signing will be done with the default key. @recips will be released
  * upon completion.
  **/
 void
-seahorse_op_encrypt_sign_file (GList *keys, SeahorseKeyPair *signer, 
-                               const gchar *path, const gchar *epath, gpgme_error_t *err)
+seahorse_op_encrypt_sign_file (GList *keys, SeahorsePGPKey *signer, const gchar *path, 
+                               const gchar *epath, gpgme_error_t *err)
 {
-    set_signer (signer);
-	encrypt_file_common (keys, path, epath, gpgme_op_encrypt_sign, err);
+    SeahorsePGPSource *psrc;
+    
+    g_return_if_fail (signer && SEAHORSE_IS_PGP_KEY (signer));
+    g_return_if_fail (seahorse_key_get_flags (SEAHORSE_KEY (signer)) & SKEY_FLAG_CAN_SIGN);
+
+    /* TODO: When other key types are supported we need to make sure they're
+       all from the same PGP source */
+    g_return_if_fail (keys && SEAHORSE_IS_PGP_KEY (keys->data));
+    psrc = SEAHORSE_PGP_SOURCE (seahorse_key_get_source (SEAHORSE_KEY (keys->data)));
+    g_return_if_fail (psrc != NULL && SEAHORSE_IS_PGP_SOURCE (psrc));
+
+    set_signer (psrc, signer);
+	encrypt_file_common (psrc, keys, path, epath, gpgme_op_encrypt_sign, err);
 }
 
 /**
@@ -509,17 +523,28 @@ seahorse_op_encrypt_sign_file (GList *keys, SeahorseKeyPair *signer,
  * @err: Optional error value
  *
  * Tries to encrypt and sign @text to @recips, saving any errors in @err.
- * Signing will be done with the default key of @sctx. @recips will be released
- * upon completion. The ASCII Armor setting of @sctx will be ignored.
+ * Signing will be done with the default key. @recips will be released
+ * upon completion.
  *
  * Returns: The encrypted and signed text or NULL if the operation fails
  **/
 gchar*
-seahorse_op_encrypt_sign_text (GList *keys, SeahorseKeyPair *signer, 
+seahorse_op_encrypt_sign_text (GList *keys, SeahorsePGPKey *signer, 
                                const gchar *text, gpgme_error_t *err)
 {
-    set_signer (signer);
-	return encrypt_text_common (keys, text, gpgme_op_encrypt_sign, err);
+    SeahorsePGPSource *psrc;
+    
+    g_return_val_if_fail (signer && SEAHORSE_IS_PGP_KEY (signer), NULL);
+    g_return_val_if_fail (seahorse_key_get_flags (SEAHORSE_KEY (signer)) & SKEY_FLAG_CAN_SIGN, NULL);
+
+    /* TODO: When other key types are supported we need to make sure they're
+       all from the same PGP source */
+    g_return_val_if_fail (keys && SEAHORSE_IS_PGP_KEY (keys->data), NULL);
+    psrc = SEAHORSE_PGP_SOURCE (seahorse_key_get_source (SEAHORSE_KEY (keys->data)));
+    g_return_val_if_fail (psrc != NULL && SEAHORSE_IS_PGP_SOURCE (psrc), NULL);
+
+    set_signer (psrc, signer);
+	return encrypt_text_common (psrc, keys, text, gpgme_op_encrypt_sign, err);
 }
 
 /**
@@ -534,7 +559,7 @@ seahorse_op_encrypt_sign_text (GList *keys, SeahorseKeyPair *signer,
  * The status of any verified signatures will be saved in @status.
  **/
 void
-seahorse_op_verify_file (SeahorseKeySource *sksrc, const gchar *path, const gchar *original,
+seahorse_op_verify_file (SeahorsePGPSource *psrc, const gchar *path, const gchar *original,
                          gpgme_verify_result_t *status, gpgme_error_t *err)
 {
 	gpgme_data_t sig, plain;
@@ -553,8 +578,8 @@ seahorse_op_verify_file (SeahorseKeySource *sksrc, const gchar *path, const gcha
 	}
  
 	/* verify sig file, release plain data */
-    *err = gpgme_op_verify (sksrc->ctx, sig, plain, NULL);
-    *status = gpgme_op_verify_result (sksrc->ctx);
+    *err = gpgme_op_verify (psrc->gctx, sig, plain, NULL);
+    *status = gpgme_op_verify_result (psrc->gctx);
     gpgme_data_release (sig); 
 	gpgme_data_release (plain);
 	g_return_if_fail (GPG_IS_OK (*err));
@@ -568,13 +593,12 @@ seahorse_op_verify_file (SeahorseKeySource *sksrc, const gchar *path, const gcha
  * @err: Optional error value
  *
  * Tries to verify @text, saving any errors in @err. The status of any verified
- * signatures will be saved in @status. The ASCII Armor setting of @sctx will
- * be ignored.
+ * signatures will be saved in @status. 
  *
  * Returns: The verified text or NULL if the operation fails
  **/
 gchar*
-seahorse_op_verify_text (SeahorseKeySource *sksrc, const gchar *text,
+seahorse_op_verify_text (SeahorsePGPSource *psrc, const gchar *text,
                          gpgme_verify_result_t *status, gpgme_error_t *err)
 {
 	gpgme_data_t sig, plain;
@@ -593,10 +617,10 @@ seahorse_op_verify_text (SeahorseKeySource *sksrc, const gchar *text,
 		g_return_val_if_reached (NULL);
 	}
 	/* verify data with armor */
-	gpgme_set_armor (sksrc->ctx, TRUE);
+	gpgme_set_armor (psrc->gctx, TRUE);
     /* verify sig file, release plain data */
-    *err = gpgme_op_verify (sksrc->ctx, sig, NULL, plain);
-    *status = gpgme_op_verify_result (sksrc->ctx);
+    *err = gpgme_op_verify (psrc->gctx, sig, NULL, plain);
+    *status = gpgme_op_verify_result (psrc->gctx);
     gpgme_data_release (sig);     
 	g_return_val_if_fail (GPG_IS_OK (*err), NULL);
 	/* return verified text */
@@ -605,14 +629,14 @@ seahorse_op_verify_text (SeahorseKeySource *sksrc, const gchar *text,
 
 /* helper function to decrypt and verify @cipher. @cipher will be released. */
 static void
-decrypt_verify_data (SeahorseKeySource *sksrc, gpgme_data_t cipher,
+decrypt_verify_data (SeahorsePGPSource *psrc, gpgme_data_t cipher,
                      gpgme_data_t plain, gpgme_verify_result_t *status, 
                      gpgme_error_t *err)
 {
-	*err = gpgme_op_decrypt_verify (sksrc->ctx, cipher, plain);
+	*err = gpgme_op_decrypt_verify (psrc->gctx, cipher, plain);
         
     if (status)
-    	*status = gpgme_op_verify_result (sksrc->ctx);
+    	*status = gpgme_op_verify_result (psrc->gctx);
      
 	gpgme_data_release (cipher);
 }
@@ -629,7 +653,7 @@ decrypt_verify_data (SeahorseKeySource *sksrc, gpgme_data_t cipher,
  * status of any verified signatures will be saved in @status. 
  **/
 void
-seahorse_op_decrypt_verify_file (SeahorseKeySource *sksrc, const gchar *path, 
+seahorse_op_decrypt_verify_file (SeahorsePGPSource *psrc, const gchar *path, 
                                  const gchar *dpath, gpgme_verify_result_t *status, 
                                  gpgme_error_t *err)
 {
@@ -649,7 +673,7 @@ seahorse_op_decrypt_verify_file (SeahorseKeySource *sksrc, const gchar *path,
     }
 
 	/* verify data */
-	decrypt_verify_data (sksrc, cipher, plain, status, err);
+	decrypt_verify_data (psrc, cipher, plain, status, err);
 
     gpgme_data_release (plain);
 }
@@ -662,13 +686,12 @@ seahorse_op_decrypt_verify_file (SeahorseKeySource *sksrc, const gchar *path,
  * @err: Optional error value
  *
  * Tries to decrypt and verify @text, saving any errors in @err. The status of
- * any verified signatures will be saved in @status. The ASCII Armor setting
- * of @sctx will be ignored.
+ * any verified signatures will be saved in @status. 
  *
  * Returns: The decrypted text or NULL if the operation fails
  **/
 gchar*
-seahorse_op_decrypt_verify_text (SeahorseKeySource *sksrc, const gchar *text,
+seahorse_op_decrypt_verify_text (SeahorsePGPSource *psrc, const gchar *text,
                                  gpgme_verify_result_t *status, gpgme_error_t *err)
 {
 	gpgme_data_t cipher, plain;
@@ -684,8 +707,8 @@ seahorse_op_decrypt_verify_text (SeahorseKeySource *sksrc, const gchar *text,
     g_return_val_if_fail (GPG_IS_OK (*err), NULL);
     
 	/* get decrypted data with armor */
-	gpgme_set_armor (sksrc->ctx, TRUE);
-	decrypt_verify_data (sksrc, cipher, plain, status, err);
+	gpgme_set_armor (psrc->gctx, TRUE);
+	decrypt_verify_data (psrc, cipher, plain, status, err);
 	g_return_val_if_fail (GPG_IS_OK (*err), NULL);
 	/* return text of decrypted data */
 	return seahorse_util_write_data_to_text (plain, TRUE);
