@@ -29,6 +29,7 @@
 #include "seahorse-pgp-key-op.h"
 #include "seahorse-util.h"
 #include "seahorse-libdialogs.h"
+#include "seahorse-pgp-operation.h"
 
 #define PROMPT "keyedit.prompt"
 #define QUIT "quit"
@@ -65,84 +66,84 @@
  * @type: Key type. Supported types are #DSA_ELGAMAL, #DSA, and #RSA_SIGN
  * @length: Length of key, must be within the range of @type specified by #SeahorseKeyLength
  * @expires: Expiration date of key
- *
+ * @err: Catches errors in the params
+ * 
  * Tries to generate a new key based on given parameters.
- * The generation operation is done with a new GPGME context
- * If the generation is successful, seahorse_key_source_refresh() will be
- * called with @sksrc.
  *
- * Returns: gpgme_error_t
+ * Returns: SeahorseOperation*
  **/
-gpgme_error_t
+SeahorseOperation*
 seahorse_pgp_key_op_generate (SeahorsePGPSource *psrc, const gchar *name,
-			  const gchar *email, const gchar *comment,
-			  const gchar *passphrase, const SeahorseKeyEncType type,
-			  const guint length, const time_t expires)
+                              const gchar *email, const gchar *comment,
+                              const gchar *passphrase, const SeahorseKeyEncType type,
+                              const guint length, const time_t expires, gpgme_error_t *err)
 {
-	gchar *common, *key_type, *start, *parms, *expires_date;
-    gpgme_error_t err;
-	gpgme_ctx_t new_ctx;
-	
-	g_return_val_if_fail (strlen (name) >= 5, GPG_E (GPG_ERR_INV_VALUE));
-	
-	/* Check lengths for each type */
-	switch (type) {
-		case DSA_ELGAMAL:
-			g_return_val_if_fail (length >= ELGAMAL_MIN && length <= LENGTH_MAX, GPG_E (GPG_ERR_INV_VALUE));
-			break;
-		case DSA:
-			g_return_val_if_fail (length >= DSA_MIN && length <= DSA_MAX, GPG_E (GPG_ERR_INV_VALUE));
-			break;
-		case RSA_SIGN:
-			g_return_val_if_fail (length >= RSA_MIN && length <= LENGTH_MAX, GPG_E (GPG_ERR_INV_VALUE));
-			break;
-		default:
-			g_return_val_if_reached (GPG_E (GPG_ERR_INV_VALUE));
-			break;
-	}
-	
-	if (expires != 0)
-		expires_date = seahorse_util_get_date_string (expires);
-	else
-		expires_date = "0";
-	
-	/* Common xml */
-	common = g_strdup_printf ("Name-Real: %s\nExpire-Date: %s\nPassphrase: %s\n"
-		"</GnupgKeyParms>", name, expires_date, passphrase);
-	if (email != NULL && strlen (email) > 0)
-		common = g_strdup_printf ("Name-Email: %s\n%s", email, common);
-	if (comment != NULL && strlen (comment) > 0)
-		common = g_strdup_printf ("Name-Comment: %s\n%s", comment, common);
-	
-	if (type == RSA_SIGN)
-		key_type = "Key-Type: RSA";
-	else
-		key_type = "Key-Type: DSA";
-	
-	start = g_strdup_printf ("<GnupgKeyParms format=\"internal\">\n%s\nKey-Length: ", key_type);
-	
-	/* Subkey xml */
-	if (type == DSA_ELGAMAL)
-		parms = g_strdup_printf ("%s%d\nSubkey-Type: ELG-E\nSubkey-Length: %d\n%s",
-					 start, DSA_MAX, length, common);
-	else
-		parms = g_strdup_printf ("%s%d\n%s", start, length, common);
-
-    new_ctx = seahorse_pgp_source_new_context (psrc);
-    g_return_val_if_fail (new_ctx != NULL, GPG_E (GPG_ERR_GENERAL));
+    SeahorsePGPOperation *pop = NULL;
+    gchar *common, *key_type, *start, *expires_date;
+    const gchar *parms;
     
-	err = gpgme_op_genkey (new_ctx, parms, NULL, NULL);
-    gpgme_release (new_ctx);
-	
-	if (GPG_IS_OK (err))
-        seahorse_key_source_load_async (SEAHORSE_KEY_SOURCE (psrc), SKSRC_LOAD_NEW, NULL);
-	
-	/* Free xmls */
-	g_free (parms);
-	g_free (start);
-	g_free (common);
-	
-	return err;
+    *err = GPG_OK;
+
+    if (strlen (name) < 5)
+        *err = GPG_E (GPG_ERR_INV_VALUE);
+
+    /* Check lengths for each type */
+    switch (type) {
+    case DSA_ELGAMAL:
+        if (length < ELGAMAL_MIN || length > LENGTH_MAX)
+            *err = GPG_E (GPG_ERR_INV_VALUE);
+        break;
+    case DSA:
+        if (length < DSA_MIN || length > DSA_MAX)
+            *err = GPG_E (GPG_ERR_INV_VALUE);
+        break;
+    case RSA_SIGN:
+        if (length < RSA_MIN || length > LENGTH_MAX)
+            *err = GPG_E (GPG_ERR_INV_VALUE);
+        break;
+    default:
+        *err = GPG_E (GPG_ERR_INV_VALUE);
+        break;
+    }
+
+    if (0 != expires)
+        expires_date = seahorse_util_get_date_string (expires);
+    else
+        expires_date = g_strdup ("0");
+
+    /* Common xml */
+    common = g_strdup_printf ("Name-Real: %s\nExpire-Date: %s\nPassphrase: %s\n"
+                              "</GnupgKeyParms>", name, expires_date, passphrase);
+    if (email != NULL && strlen (email) > 0)
+        common = g_strdup_printf ("Name-Email: %s\n%s", email, common);
+    if (comment != NULL && strlen (comment) > 0)
+        common = g_strdup_printf ("Name-Comment: %s\n%s", comment, common);
+
+    if (type == RSA_SIGN)
+        key_type = "Key-Type: RSA";
+    else
+        key_type = "Key-Type: DSA";
+
+    start = g_strdup_printf ("<GnupgKeyParms format=\"internal\">\n%s\nKey-Length: ", key_type);
+
+    /* Subkey xml */
+    if (type == DSA_ELGAMAL)
+        parms = g_strdup_printf ("%s%d\nSubkey-Type: ELG-E\nSubkey-Length: %d\n%s",
+                                 start, DSA_MAX, length, common);
+    else
+        parms = g_strdup_printf ("%s%d\n%s", start, length, common);
+
+    if (GPG_IS_OK (*err)) {
+        pop = seahorse_pgp_operation_new (NULL);
+        *err = gpgme_op_genkey_start (pop->gctx, parms, NULL, NULL);
+     }
+
+    /* Free xmls */
+    g_free (start);
+    g_free (common);
+    g_free (expires_date);
+
+    return pop ? SEAHORSE_OPERATION (pop) : NULL;
 }
 
 /* helper function for deleting @skey */
@@ -261,27 +262,39 @@ seahorse_pgp_key_op_edit (gpointer data, gpgme_status_code_t status,
 }
 
 /* Common edit operation */
+static gpgme_error_t
+edit_gpgme_key (SeahorsePGPSource *psrc, gpgme_key_t key, SeahorseEditParm *parms)
+{
+    gpgme_data_t out;
+    gpgme_error_t err;
+    
+    g_assert (psrc && SEAHORSE_IS_PGP_SOURCE (psrc));
+    
+    err = gpgme_data_new (&out);
+    g_return_val_if_fail (GPG_IS_OK (err), err);
+    
+    /* do edit callback, release data */
+    err = gpgme_op_edit (psrc->gctx, key, seahorse_pgp_key_op_edit, parms, out);
+    gpgme_data_release (out);
+    
+    return err;
+}
+
 static gpgme_error_t
 edit_key (SeahorsePGPKey *pkey, SeahorseEditParm *parms, SeahorseKeyChange change)
 {
     SeahorsePGPSource *psrc;
-	gpgme_data_t out;
-	gpgme_error_t err;
-	
+    gpgme_error_t err;
+    
     psrc = SEAHORSE_PGP_SOURCE (seahorse_key_get_source (SEAHORSE_KEY (pkey)));
     g_return_val_if_fail (psrc && SEAHORSE_IS_PGP_SOURCE (psrc), GPG_E (GPG_ERR_INV_KEYRING));
   
-	err = gpgme_data_new (&out);
-	g_return_val_if_fail (GPG_IS_OK (err), err);
+    err = edit_gpgme_key (psrc, pkey->pubkey, parms);
+
+    if (GPG_IS_OK (err) && (change != 0))
+        seahorse_key_changed (SEAHORSE_KEY (pkey), change);
     
-	/* do edit callback, release data */
-	err = gpgme_op_edit (psrc->gctx, pkey->pubkey, seahorse_pgp_key_op_edit, parms, out);
-	gpgme_data_release (out);
-	
-	if (GPG_IS_OK (err) && (change != 0))
-    	seahorse_key_changed (SEAHORSE_KEY (pkey), change);
-    
-	return err;
+    return err;
 }
 
 typedef struct
@@ -460,36 +473,50 @@ sign_transit (guint current_state, gpgme_status_code_t status,
  * Returns: Error value
  **/
 gpgme_error_t
-seahorse_pgp_key_op_sign (SeahorsePGPKey *pkey, const guint index,
-		              SeahorseSignCheck check, SeahorseSignOptions options)
+seahorse_pgp_key_op_sign (SeahorsePGPKey *pkey, SeahorsePGPKey *signer, 
+                          const guint index, SeahorseSignCheck check, 
+                          SeahorseSignOptions options)
 {
-	SignParm *sign_parm;
-	SeahorseEditParm *parms;
+    SeahorseKeySource *sksrc;
+    SignParm *sign_parm;
+    SeahorseEditParm *parms;
     gpgme_error_t err;
-	guint real_index = seahorse_pgp_key_get_actual_uid(pkey, index);
-	guint num_userids = seahorse_pgp_key_get_num_userids (pkey);
-	guint num_photoids = seahorse_pgp_key_get_num_photoids (pkey);
-	
-	g_return_val_if_fail (SEAHORSE_IS_PGP_KEY (pkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
-	DEBUG_OPERATION(("SignKey: index = %i,real_index = %i, num_userids = %i, num_photoids = %i\n", index, real_index, num_userids, num_photoids));
-	g_return_val_if_fail (real_index <= (num_userids + num_photoids), GPG_E (GPG_ERR_INV_VALUE));
-	
-	sign_parm = g_new (SignParm, 1);
-	sign_parm->index = real_index;
-	sign_parm->expire = ((options & SIGN_EXPIRES) != 0);
-	sign_parm->check = check;
-	sign_parm->command = "sign";
-	
-	/* if sign is local */
-	if ((options & SIGN_LOCAL) != 0)
-		sign_parm->command = g_strdup_printf ("l%s", sign_parm->command);
-	/* if sign is non-revocable */
-	if ((options & SIGN_NO_REVOKE) != 0)
-		sign_parm->command = g_strdup_printf ("nr%s", sign_parm->command);
-	
-	parms = seahorse_edit_parm_new (SIGN_START, sign_action, sign_transit, sign_parm);
-	
-	err =  edit_key (pkey, parms, SKEY_CHANGE_SIGNERS);
+    guint real_index = seahorse_pgp_key_get_actual_uid(pkey, index);
+    guint num_userids = seahorse_pgp_key_get_num_userids (pkey);
+    guint num_photoids = seahorse_pgp_key_get_num_photoids (pkey);
+    
+    g_return_val_if_fail (SEAHORSE_IS_PGP_KEY (pkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
+    DEBUG_OPERATION(("SignKey: index = %i,real_index = %i, num_userids = %i, num_photoids = %i\n", index, real_index, num_userids, num_photoids));
+    g_return_val_if_fail (real_index <= (num_userids + num_photoids), GPG_E (GPG_ERR_INV_VALUE));
+    
+    sign_parm = g_new (SignParm, 1);
+    sign_parm->index = real_index;
+    sign_parm->expire = ((options & SIGN_EXPIRES) != 0);
+    sign_parm->check = check;
+
+    sign_parm->command = g_strdup_printf ("%s%ssign", 
+                                (options & SIGN_NO_REVOKE) ? "nr" : "",
+                                (options & SIGN_LOCAL) ? "l" : "");
+    
+    if (signer) {
+        sksrc = seahorse_key_get_source (SEAHORSE_KEY (pkey));
+        
+        g_assert (SEAHORSE_IS_PGP_SOURCE (sksrc));
+        g_assert (SEAHORSE_PGP_SOURCE (sksrc)->gctx);
+        
+        gpgme_signers_clear (SEAHORSE_PGP_SOURCE (sksrc)->gctx);
+
+        g_return_val_if_fail (signer->seckey != NULL, GPG_E (GPG_ERR_INV_VALUE));
+        err = gpgme_signers_add (SEAHORSE_PGP_SOURCE (sksrc)->gctx, signer->seckey);
+        if (!GPG_IS_OK (err))
+            return err;
+    }
+
+    
+    parms = seahorse_edit_parm_new (SIGN_START, sign_action, sign_transit, sign_parm);
+    
+    err =  edit_key (pkey, parms, SKEY_CHANGE_SIGNERS);
+    g_free (sign_parm->command);
  
     /* If it was already signed then it's not an error */
     if (!GPG_IS_OK (err) && gpgme_err_code (err) == GPG_ERR_EALREADY)
@@ -2075,7 +2102,7 @@ seahorse_pgp_key_op_del_uid (SeahorsePGPKey *pkey, const guint index)
 }
 
 typedef struct {
-	gchar *filename;
+    const gchar *filename;
 } PhotoIdAddParm;
 
 typedef enum {
@@ -2093,46 +2120,26 @@ static gpgme_error_t
 photoid_add_action (guint state, gpointer data, int fd)
 {
     PhotoIdAddParm *parm = (PhotoIdAddParm*)data;
-    GtkWidget *question, *delete_button, *cancel_button;
-    gint response;
     
     switch (state) {
-	    case PHOTO_ID_ADD_COMMAND:
-            PRINT ((fd, "addphoto"));
-	        break;
-	    case PHOTO_ID_ADD_URI:
-            PRINT ((fd, parm->filename));
-	    	break;
-	    case PHOTO_ID_ADD_BIG:
-	    	/* let the user know their file is big and ask them if they still want to use it */
-	    	question = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
-		                        _("This photo ID file is very large. Are you sure you want to add it to your key?"));
-		    
-		    delete_button = gtk_button_new_from_stock (GTK_STOCK_ADD);
-		    cancel_button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
-
-		    /* add widgets to action area */
-		    gtk_dialog_add_action_widget (GTK_DIALOG(question), GTK_WIDGET (cancel_button), GTK_RESPONSE_REJECT);
-		    gtk_dialog_add_action_widget (GTK_DIALOG(question), GTK_WIDGET (delete_button), GTK_RESPONSE_ACCEPT);
-		   
-		    /* show widgets */
-		    gtk_widget_show (delete_button);
-		    gtk_widget_show (cancel_button);
-		       
-		    response = gtk_dialog_run (GTK_DIALOG (question));
-		    gtk_widget_destroy (question);
-		    
-	        PRINT ((fd, (response == GTK_RESPONSE_ACCEPT) ? YES : NO));
-	        break;
-	    case PHOTO_ID_ADD_QUIT:
-            PRINT ((fd, QUIT));
-	        break;
-	    case PHOTO_ID_ADD_SAVE:
-            PRINT ((fd, YES));
-	        break;
-	    default:
-	        g_return_val_if_reached (GPG_E (GPG_ERR_GENERAL));
-	        break;
+    case PHOTO_ID_ADD_COMMAND:
+        PRINT ((fd, "addphoto"));
+        break;
+    case PHOTO_ID_ADD_URI:
+        PRINT ((fd, parm->filename));
+        break;
+    case PHOTO_ID_ADD_BIG:
+        PRINT ((fd, YES));
+        break;
+    case PHOTO_ID_ADD_QUIT:
+        PRINT ((fd, QUIT));
+        break;
+    case PHOTO_ID_ADD_SAVE:
+        PRINT ((fd, YES));
+        break;
+    default:
+        g_return_val_if_reached (GPG_E (GPG_ERR_GENERAL));
+        break;
     }
   
     seahorse_util_print_fd (fd, "\n");
@@ -2212,7 +2219,7 @@ photoid_add_transit (guint current_state, gpgme_status_code_t status,
  * Returns: Error value
  **/
 gpgme_error_t 
-seahorse_pgp_key_op_photoid_add	(SeahorsePGPKey *pkey, gchar *filename)
+seahorse_pgp_key_op_photoid_add	(SeahorsePGPKey *pkey, const gchar *filename)
 {
 	SeahorseEditParm *parms;
 	PhotoIdAddParm *photoid_add_parm;
@@ -2436,70 +2443,71 @@ photoid_load_transit (guint current_state, gpgme_status_code_t status,
  * Returns: Error value
  **/
 gpgme_error_t 
-seahorse_pgp_key_op_photoid_load (SeahorsePGPKey *pkey, gpgmex_photo_id_t *photoids)
+seahorse_pgp_key_op_photoid_load (SeahorsePGPSource *psrc, gpgme_key_t key, 
+                                  gpgmex_photo_id_t *photoids)
 {
-	SeahorseEditParm *parms;
-	PhotoIdLoadParm *photoid_load_parm;
-	gpgme_error_t err;
+    /* Make sure there's enough room for the .jpg extension */
+    gchar image_path[]    = "/tmp/seahorse-photoid-XXXXXX\0\0\0\0";
+    
+    SeahorseEditParm *parms;
+    PhotoIdLoadParm *photoid_load_parm;
+    gpgme_error_t err;
     const gchar *oldpath;
     gchar *path;
-	guint fd;
-
-    DEBUG_OPERATION (("PhotoIDLoad Start\n"));     
+    guint fd;
 
     g_return_val_if_fail(*photoids == NULL, GPG_E (GPG_ERR_GENERAL));
-    
-    /* Make sure there's enough room for the .jpg extension */
-    gchar image_path[] = "/tmp/seahorse-photoid-XXXXXX\0\0\0\0";
-    
-    g_return_val_if_fail (SEAHORSE_IS_PGP_KEY (pkey), GPG_E(GPG_ERR_WRONG_KEY_USAGE));
-    
- 	fd = g_mkstemp (image_path);
- 	
- 	if(fd == -1)
- 		err = GPG_E(GPG_ERR_GENERAL);
-    
- 	else {
+    g_return_val_if_fail (SEAHORSE_IS_PGP_SOURCE (psrc), 
+                          GPG_E (GPG_ERR_GENERAL));
+    g_return_val_if_fail (key && seahorse_pgp_key_get_id (key, 0), 
+                          GPG_E (GPG_ERR_WRONG_KEY_USAGE));
 
-		g_unlink(image_path);
- 		close(fd);
- 		strcat (image_path, ".jpg");
-	    
-		photoid_load_parm = g_new0 (PhotoIdLoadParm, 1);
-		photoid_load_parm->uid = 1;
-		photoid_load_parm->num_uids = 0;
-		photoid_load_parm->list = *photoids;
-		photoid_load_parm->first = NULL;
-		photoid_load_parm->output_file = image_path;
-		
-		DEBUG_OPERATION (("PhotoIdLoad KeyID %s\n", seahorse_key_get_keyid (SEAHORSE_KEY (pkey))));
-        err = gpgmex_op_num_uids (NULL, 
-                                  seahorse_key_get_keyid (SEAHORSE_KEY (pkey)), 
-							      &(photoid_load_parm->num_uids));
+    DEBUG_OPERATION (("PhotoIDLoad Start\n"));
+    
+    fd = g_mkstemp (image_path);
+    if(fd == -1) 
+        err = GPG_E(GPG_ERR_GENERAL);
+    
+    else {
+
+        g_unlink(image_path);
+        close(fd);
+        strcat (image_path, ".jpg");
+        
+        photoid_load_parm = g_new0 (PhotoIdLoadParm, 1);
+        photoid_load_parm->uid = 1;
+        photoid_load_parm->num_uids = 0;
+        photoid_load_parm->list = *photoids;
+        photoid_load_parm->first = NULL;
+        photoid_load_parm->output_file = image_path;
+        
+        DEBUG_OPERATION (("PhotoIdLoad KeyID %s\n", seahorse_pgp_key_get_id (key, 0)));
+        err = gpgmex_op_num_uids (NULL, seahorse_pgp_key_get_id (key, 0),
+                                  &(photoid_load_parm->num_uids));
         DEBUG_OPERATION (("PhotoIDLoad Number of UIDs %i\n", photoid_load_parm->num_uids));
         
-		if (GPG_IS_OK(err)) {
+        if (GPG_IS_OK(err)) {
             
-			setenv("SEAHORSE_IMAGE_FILE", image_path, 1);
-			oldpath = getenv("PATH");
+            setenv("SEAHORSE_IMAGE_FILE", image_path, 1);
+            oldpath = getenv("PATH");
             
             path = g_strdup_printf ("%s:%s", EXECDIR, getenv("PATH"));
-			setenv("PATH", path, 1);
+            setenv("PATH", path, 1);
             g_free (path);
-			
-			parms = seahorse_edit_parm_new (PHOTO_ID_LOAD_START, photoid_load_action,
-							                photoid_load_transit, photoid_load_parm);
-			
-			/* generate list */
-			err = edit_key (pkey, parms, 0);
+            
+            parms = seahorse_edit_parm_new (PHOTO_ID_LOAD_START, photoid_load_action,
+                                            photoid_load_transit, photoid_load_parm);
+            
+            /* generate list */
+            err = edit_gpgme_key (psrc, key, parms);
             setenv("PATH", oldpath, 1);
             
-			*photoids = photoid_load_parm->first;  
-		}
- 	}
- 	
- 	DEBUG_OPERATION (("PhotoIDLoad Done\n"));
- 	
+            *photoids = photoid_load_parm->first;  
+        }
+    }
+    
+    DEBUG_OPERATION (("PhotoIDLoad Done\n"));
+    
     return err;
 }
 
