@@ -42,31 +42,29 @@
 
 #include "seahorse-context.h"
 
-#define SEAHORSE_TYPE_WIDGET		(seahorse_widget_get_type ())
-#define SEAHORSE_WIDGET(obj)		(GTK_CHECK_CAST ((obj), SEAHORSE_TYPE_WIDGET, SeahorseWidget))
-#define SEAHORSE_WIDGET_CLASS(klass)	(GTK_CHECK_CLASS_CAST ((klass), SEAHORSE_TYPE_WIDGET, SeahorseWidgetClass))
-#define SEAHORSE_IS_WIDGET(obj)		(GTK_CHECK_TYPE ((obj), SEAHORSE_TYPE_WIDGET))
-#define SEAHORSE_IS_WIDGET_CLASS(klass)	(GTK_CHECK_CLASS_TYPE ((klass), SEAHORSE_TYPE_WIDGET))
-#define SEAHORSE_WIDGET_GET_CLASS(obj)	(GTK_CHECK_GET_CLASS ((obj), SEAHORSE_TYPE_WIDGET, SeahorseWidgetClass))
+#define SEAHORSE_TYPE_WIDGET            (seahorse_widget_get_type ())
+#define SEAHORSE_WIDGET(obj)            (GTK_CHECK_CAST ((obj), SEAHORSE_TYPE_WIDGET, SeahorseWidget))
+#define SEAHORSE_WIDGET_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), SEAHORSE_TYPE_WIDGET, SeahorseWidgetClass))
+#define SEAHORSE_IS_WIDGET(obj)         (GTK_CHECK_TYPE ((obj), SEAHORSE_TYPE_WIDGET))
+#define SEAHORSE_IS_WIDGET_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), SEAHORSE_TYPE_WIDGET))
+#define SEAHORSE_WIDGET_GET_CLASS(obj)  (GTK_CHECK_GET_CLASS ((obj), SEAHORSE_TYPE_WIDGET, SeahorseWidgetClass))
 
 typedef struct _SeahorseWidget SeahorseWidget;
 typedef struct _SeahorseWidgetClass SeahorseWidgetClass;
 
-struct _SeahorseWidget
-{
-	GObject		parent;
-	
-	/*< public >*/
-	GladeXML	*xml;
-	gchar		*name;
+struct _SeahorseWidget {
+    GtkObject parent;
+
+    /*< public >*/
+    GladeXML *xml;
+    gchar *name;
     
     /*< private >*/
     GtkUIManager   *ui;
 };
 
-struct _SeahorseWidgetClass
-{
-	GObjectClass	parent_class;
+struct _SeahorseWidgetClass {
+    GtkObjectClass parent_class;
 };
 
 GType            seahorse_widget_get_type ();
@@ -77,10 +75,17 @@ SeahorseWidget*  seahorse_widget_new_allow_multiple (gchar            *name);
 
 GtkWidget*       seahorse_widget_get_top            (SeahorseWidget   *swidget);
 
+GtkWidget*       seahorse_widget_get_widget         (SeahorseWidget   *swidget,
+                                                     const char       *identifier);
+
 void             seahorse_widget_show               (SeahorseWidget   *swidget);
 
 void             seahorse_widget_show_help          (SeahorseWidget   *swidget);
 
+void             seahorse_widget_set_visible        (SeahorseWidget   *swidget,
+                                                     const char       *identifier,
+                                                     gboolean         visible);
+
 void             seahorse_widget_destroy            (SeahorseWidget   *swidget);
 
 GtkWidget*       seahorse_widget_get_ui_widget      (SeahorseWidget *swidget, 
