@@ -24,8 +24,6 @@
 
 #include "seahorse-context.h"
 
-#define SEAHORSE_SCHEMAS "/apps/seahorse"
-
 #define LISTING_SCHEMAS SEAHORSE_SCHEMAS "/listing"
 #define SHOW_VALIDITY_KEY LISTING_SCHEMAS "/show_validity"
 #define SHOW_EXPIRES_KEY LISTING_SCHEMAS "/show_expires"
@@ -34,14 +32,7 @@
 #define SHOW_TYPE_KEY LISTING_SCHEMAS "/show_type"
 
 #define UI_SCHEMAS SEAHORSE_SCHEMAS "/ui"
-#define TOOLBAR_STYLE_KEY UI_SCHEMAS "/toolbar_style"
 
-#define TOOLBAR_DEFAULT "default"
-#define TOOLBAR_BOTH "both"
-#define TOOLBAR_BOTH_HORIZ "both_horiz"
-#define TOOLBAR_ICONS "icons"
-#define TOOLBAR_TEXT "text"
-
-void		seahorse_preferences_show		(SeahorseContext	*sctx);
+void		seahorse_preferences_show		(const gchar        *tabid);
 
 #endif /* __SEAHORSE_PREFERENCES_H__ */
